﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml.Serialization;

namespace DiscuzHelper
{
    public class Util
    {
        private const string LINE = "\r\n";

        private static readonly Dictionary<int, XmlSerializer> serializer_dict = new Dictionary<int, XmlSerializer>();

        private static readonly ReaderWriterLock _lock = new ReaderWriterLock();

        private static readonly object _listLock = new object();


        //private DNTParam VersionParam = DNTParam.Create("v", "1.0");
        private readonly string api_key;
        private string secret;

        public Util(string api_key, string secret, string url)
        {
            this.api_key = api_key;
            this.secret = secret;
            this.Url = url;
        }

        private static XmlSerializer ErrorSerializer
        {
            get { return GetSerializer(typeof (Error)); }
        }

        public bool UseJson { get; set; }

        internal string SharedSecret
        {
            get { return secret; }
            set { secret = value; }
        }

        internal string ApiKey
        {
            get { return api_key; }
        }

        internal string Url { get; set; }

        public T GetResponse<T>(string method_name, params DiscuzParam[] parameters)
        {
            //string url = FormatGetUrl(method_name, parameters);
            //byte[] response_bytes = GetResponseBytes(url);

            DiscuzParam[] signed = Sign(method_name, parameters);

            var builder = new StringBuilder();

            for (int i = 0; i < signed.Length; i++)
            {
                if (i > 0)
                    builder.Append("&");

                builder.Append(signed[i].ToEncodedString());
            }

            byte[] response_bytes = GetResponseBytes(Url, method_name, builder.ToString());


            XmlSerializer response_serializer = GetSerializer(typeof (T));
            try
            {
                var response = (T) response_serializer.Deserialize(new MemoryStream(response_bytes));
                return response;
            }
            catch
            {
                var error = (Error) ErrorSerializer.Deserialize(new MemoryStream(response_bytes));
                throw new DiscuzException(error.ErrorCode, error.ErrorMsg);
            }
        }


        public static byte[] GetResponseBytes(string apiUrl, string method_name, string postData)
        {
            var request = (HttpWebRequest) WebRequest.Create(apiUrl);
            request.Method = "POST";
            request.ContentType = "application/x-www-form-urlencoded; charset=utf-8";
            // request.ContentLength = postData.Length;
            request.Timeout = 20000;
            byte[] byteArray = Encoding.UTF8.GetBytes(postData);
            request.ContentLength = byteArray.Length;
            HttpWebResponse response = null;

            try
            {
                using (Stream swRequestWriter = request.GetRequestStream())
                {
                    swRequestWriter.Write(byteArray, 0, byteArray.Length);
                }

                response = (HttpWebResponse) request.GetResponse();
                using (var reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    return Encoding.UTF8.GetBytes(reader.ReadToEnd());
                }
            }
            catch (WebException ex)
            {
                throw new ApplicationException(ex.Response.ToString());
            }
            finally
            {
                if (response != null)
                    response.Close();
            }
        }


        private string FormatGetUrl(string method_name, params DiscuzParam[] parameters)
        {
            DiscuzParam[] signed = Sign(method_name, parameters);

            var builder = new StringBuilder(Url);

            for (int i = 0; i < signed.Length; i++)
            {
                if (i > 0)
                    builder.Append("&");

                builder.Append(signed[i]);
            }

            return builder.ToString();
        }

        public static XmlSerializer GetSerializer(Type t)
        {
            int type_hash = t.GetHashCode();
            const int timeout = 5000;
            try
            {
                _lock.AcquireReaderLock(timeout);
                if (!serializer_dict.ContainsKey(type_hash))
                {
                    _lock.UpgradeToWriterLock(timeout);
                    if (!serializer_dict.ContainsKey(type_hash))
                    {
                        serializer_dict.Add(type_hash, new XmlSerializer(t));
                    }
                }
                return serializer_dict[type_hash];
            }
            catch (ApplicationException ex)
            {
                throw new Exception("Accquire lock failed.", ex);
            }
            finally
            {
                if (_lock.IsReaderLockHeld)
                {
                    _lock.ReleaseReaderLock();
                }
                else if (_lock.IsWriterLockHeld)
                {
                    _lock.ReleaseWriterLock();
                }
            }
        }

        public DiscuzParam[] Sign(string method_name, DiscuzParam[] parameters)
        {
            var list = new List<DiscuzParam>(parameters);
            lock (_listLock)
            {
                list.Remove(list.FirstOrDefault(x => x.Name == "call_id"));
                list.Add(DiscuzParam.Create("call_id", DateTime.Now.Ticks));
            }


            list.Add(DiscuzParam.Create("method", method_name));
            list.Add(DiscuzParam.Create("api_key", api_key));
            list.Sort();

            var values = new StringBuilder();

            foreach (DiscuzParam param in list)
            {
                if (!string.IsNullOrEmpty(param.Value))
                    values.Append(param);
            }

            values.Append(secret);

            byte[] md5_result = MD5.Create().ComputeHash(Encoding.UTF8.GetBytes(values.ToString()));

            var sig_builder = new StringBuilder();

            foreach (byte b in md5_result)
                sig_builder.Append(b.ToString("x2"));

            list.Add(DiscuzParam.Create("sig", sig_builder.ToString()));

            return list.ToArray();
        }

        public static int GetIntFromString(string input)
        {
            try
            {
                return int.Parse(input);
            }
            catch
            {
                return 0;
            }
        }

        public static bool GetBoolFromString(string input)
        {
            try
            {
                return bool.Parse(input);
            }
            catch
            {
                return false;
            }
        }

        public static string RemoveJsonNull(string json)
        {
            json = Regex.Replace(json, @",""\w*"":null", string.Empty);
            json = Regex.Replace(json, @"""\w*"":null,", string.Empty);
            json = Regex.Replace(json, @"""\w*"":null", string.Empty);
            json = Regex.Replace(json, @",""\w*"":0", string.Empty);
            json = Regex.Replace(json, @"""\w*"":0,", string.Empty);
            return json;
        }

        /// <summary>
        ///     MD5函数
        /// </summary>
        /// <param name="str">原始字符串</param>
        /// <param name="isMD5Password">是否加密为Md5</param>
        /// <returns>MD5结果</returns>
        public string GetMD5Password(string str, bool isMD5Password)
        {
            if (isMD5Password) return str;
            byte[] b = Encoding.UTF8.GetBytes(str);
            b = new MD5CryptoServiceProvider().ComputeHash(b);
            string ret = "";
            for (int i = 0; i < b.Length; i++)
                ret += b[i].ToString("x").PadLeft(2, '0');
            return ret;
        }
    }
}